@@ -1,5 +1,3 @@
-#if (__cplusplus >= 201103L)
-
 #include "bt2_locks.h"
 
 void mcs_lock::lock(mcs_node &node) {
@@ -36,5 +34,3 @@ void spin_lock::lock() {
 void spin_lock::unlock() {
 	flag.clear(std::memory_order_release);
 }
-
-#endif
