@@ -481,6 +481,8 @@ void CFilePatternSource::open() {
 	while(filecur_ < infiles_.size()) {
 		if(infiles_[filecur_] == "-") {
 			int fd = dup(fileno(stdin));
+			SET_BINARY_MODE(fd);
+
 			if (pp_.format == BAM) {
 				compressionType_ = CompressionType::NONE;
 				fp_ = fdopen(fd, "rb");
@@ -511,16 +513,20 @@ void CFilePatternSource::open() {
 
 			bool err = false;
                         if (pp_.format == BAM) {
+				SET_BINARY_MODE(fd);
 				err = CHECK_ERROR(fp_ = fdopen(fd, "rb"));
 				compressionType_ = CompressionType::NONE;
                         } else if (is_fifo) {
+				SET_BINARY_MODE(fd);
 				err = CHECK_ERROR(zfp_ = gzdopen(fd, "rb"));
 				compressionType_ = CompressionType::GZIP;
                         } else if (is_gzipped_file(fd)) {
+				SET_BINARY_MODE(fd);
 				err = CHECK_ERROR(zfp_ = gzdopen(fd, "rb"));
 				compressionType_ = CompressionType::GZIP;
 #ifdef WITH_ZSTD
                         } else if (is_zstd_file(fd)) {
+				SET_BINARY_MODE(fd);
 				err = CHECK_ERROR(zstdfp_ = zstdFdOpen(fd));
 				compressionType_ = CompressionType::ZSTD;
 #endif
@@ -656,7 +662,7 @@ bool VectorPatternSource::parse(Read& ra, Read& rb, TReadId rdid) const {
 	int c = '\t';
 	size_t cur = 0;
 	const size_t buflen = ra.readOrigBuf.length();
-	
+
 	// Loop over the two ends
 	for(int endi = 0; endi < 2 && c == '\t'; endi++) {
 		Read& r = ((endi == 0) ? ra : rb);
@@ -702,7 +708,7 @@ bool VectorPatternSource::parse(Read& ra, Read& rb, TReadId rdid) const {
 		r.trimmed5 = (int)(nchar - r.patFw.length());
 		// record amt trimmed from 3' end due to --trim3
 		r.trimmed3 = (int)(r.patFw.trimEnd(pp_.trim3));
-		
+
 		// Parse qualities
 		assert(r.qual.empty());
 		c = ra.readOrigBuf[cur++];
@@ -786,7 +792,7 @@ pair<bool, int> FastaPatternSource::nextBatchFromFile(
 
 	}
 	// Immediate EOF case
-	if(done && (*readbuf)[readi-1].readOrigBuf.length() == 1) {
+	if(done && readi > 0 && (*readbuf)[readi-1].readOrigBuf.length() == 1) {
 		readi--;
 	}
 	return make_pair(done, readi);
@@ -969,7 +975,7 @@ bool FastaContinuousPatternSource::parse(
 	int c = '\t';
 	size_t cur = 0;
 	const size_t buflen = ra.readOrigBuf.length();
-	
+
 	// Parse read name
 	c = ra.readOrigBuf[cur++];
 	while(c != '\t' && cur < buflen) {
@@ -998,7 +1004,7 @@ bool FastaContinuousPatternSource::parse(
 	ra.trimmed5 = (int)(nchar - ra.patFw.length());
 	// record amt trimmed from 3' end due to --trim3
 	ra.trimmed3 = (int)(ra.patFw.trimEnd(pp_.trim3));
-	
+
 	// Make fake qualities
 	assert(ra.qual.empty());
 	const size_t len = ra.patFw.length();
@@ -1038,32 +1044,38 @@ pair<bool, int> FastqPatternSource::nextBatchFromFile(
 		(*readbuf)[readi].readOrigBuf.append('@');
 	}
 
-	bool done = false, aborted = false;
+	bool done = false, previous_was_newline = false;
 	// Read until we run out of input or until we've filled the buffer
 	while (readi < pt.max_buf_ && !done) {
 		Read::TBuf& buf = (*readbuf)[readi].readOrigBuf;
 		int newlines = 4;
 		while(newlines) {
 			c = getc_wrapper();
+			if (previous_was_newline && (c == '\n' || c == '\r'))
+				continue;
+			// We've encountered a new record implying that the
+			// previous record was incomplete. Reset the line count
+			// and let the parser take care of that.
+			if (previous_was_newline && c == '@' && newlines > 1) {
+				newlines = 4;
+			}
+			previous_was_newline = false;
 			done = c < 0;
 			if(c == '\n' || (done && newlines == 1)) {
 				// Saw newline, or EOF that we're
 				// interpreting as final newline
 				newlines--;
 				c = '\n';
-			} else if(done) {
-				// account for newline at the end of the file
-				if (newlines == 4) {
-					newlines = 0;
-				}
-				else {
-					aborted = true; // Unexpected EOF
-				}
+				previous_was_newline = true;
+			}
+			if(done) {
 				break;
 			}
 			buf.append(c);
 		}
-		if (c > 0) {
+		// Accept potentially incomplete or empty reads. Let the parser
+		// take care of validating those reads.
+		if (newlines < 4) {
 			if (interleaved_) {
 				// alternate between read buffers
 				batch_a = !batch_a;
@@ -1076,9 +1088,7 @@ pair<bool, int> FastqPatternSource::nextBatchFromFile(
 			}
 		}
 	}
-	if(aborted) {
-		readi--;
-	}
+
 	return make_pair(done, readi);
 }
 
@@ -1108,7 +1118,7 @@ bool FastqPatternSource::parse(Read &r, Read& rb, TReadId rdid) const {
 		}
 		r.name.append(c);
 	}
-	
+
 	// Parse sequence
 	int nchar = 0;
 	assert(r.patFw.empty());
@@ -1127,7 +1137,7 @@ bool FastqPatternSource::parse(Read &r, Read& rb, TReadId rdid) const {
 	}
 	r.trimmed5 = (int)(nchar - r.patFw.length());
 	r.trimmed3 = (int)(r.patFw.trimEnd(pp_.trim3));
-	
+
 	assert_eq('+', c);
 	do {
 		assert(cur < r.readOrigBuf.length());
@@ -1136,7 +1146,7 @@ bool FastqPatternSource::parse(Read &r, Read& rb, TReadId rdid) const {
 	while(cur < buflen && (c == '\n' || c == '\r')) {
 		c = r.readOrigBuf[cur++];
 	}
-	
+
 	assert(r.qual.empty());
 	if(nchar > 0) {
 		int nqual = 0;
@@ -1194,6 +1204,7 @@ bool FastqPatternSource::parse(Read &r, Read& rb, TReadId rdid) const {
 	if(!rb.parsed && !rb.readOrigBuf.empty()) {
 		return parse(rb, r, rdid);
 	}
+
 	return true;
 }
 
@@ -1278,7 +1289,7 @@ std::pair<bool, int> BAMPatternSource::nextBatch(PerThreadReadBuf& pt, bool batc
 	uint16_t cdata_len;
 	unsigned nread = 0;
 
-	ThreadSafe ts(mutex);
+	// ThreadSafe ts(mutex);
 	do {
 		if (alignment_offset >= alignment_batch.size()) {
 			BGZF block;
@@ -1495,7 +1506,7 @@ bool TabbedPatternSource::parse(Read& ra, Read& rb, TReadId rdid) const {
 	int c = '\t';
 	size_t cur = 0;
 	const size_t buflen = ra.readOrigBuf.length();
-	
+
 	// Loop over the two ends
 	for(int endi = 0; endi < 2 && c == '\t'; endi++) {
 		Read& r = ((endi == 0) ? ra : rb);
@@ -1541,7 +1552,7 @@ bool TabbedPatternSource::parse(Read& ra, Read& rb, TReadId rdid) const {
 		r.trimmed5 = (int)(nchar - r.patFw.length());
 		// record amt trimmed from 3' end due to --trim3
 		r.trimmed3 = (int)(r.patFw.trimEnd(pp_.trim3));
-		
+
 		// Parse qualities
 		assert(r.qual.empty());
 		c = ra.readOrigBuf[cur++];
@@ -1649,12 +1660,12 @@ bool RawPatternSource::parse(Read& r, Read& rb, TReadId rdid) const {
 	r.trimmed5 = (int)(nchar - r.patFw.length());
 	// record amt trimmed from 3' end due to --trim3
 	r.trimmed3 = (int)(r.patFw.trimEnd(pp_.trim3));
-	
+
 	// Give the name field a dummy value
 	char cbuf[20];
 	itoa10<TReadId>(rdid, cbuf);
 	r.name.install(cbuf);
-	
+
 	// Give the base qualities dummy values
 	assert(r.qual.empty());
 	const size_t len = r.patFw.length();
@@ -1699,28 +1710,29 @@ std::pair<bool, int> SRAPatternSource::nextBatchImpl(
 	size_t readi = 0;
 	bool done = false;
 
-	for(; readi < pt.max_buf_; readi++) {
-		if(!sra_its_[pt.tid_]->nextRead() || !sra_its_[pt.tid_]->nextFragment()) {
+	while (readi < pt.max_buf_) {
+		if(!read_iter_->nextRead() || !read_iter_->nextFragment()) {
 			done = true;
 			break;
 		}
-		const ngs::StringRef rname = sra_its_[pt.tid_]->getReadId();
-		const ngs::StringRef ra_seq = sra_its_[pt.tid_]->getFragmentBases();
-		const ngs::StringRef ra_qual = sra_its_[pt.tid_]->getFragmentQualities();
+		const ngs::StringRef rname = read_iter_->getReadId();
+		const ngs::StringRef ra_seq = read_iter_->getFragmentBases();
+		const ngs::StringRef ra_qual = read_iter_->getFragmentQualities();
 		readbuf[readi].readOrigBuf.install(rname.data(), rname.size());
 		readbuf[readi].readOrigBuf.append('\t');
 		readbuf[readi].readOrigBuf.append(ra_seq.data(), ra_seq.size());
 		readbuf[readi].readOrigBuf.append('\t');
 		readbuf[readi].readOrigBuf.append(ra_qual.data(), ra_qual.size());
-		if(sra_its_[pt.tid_]->nextFragment()) {
-			const ngs::StringRef rb_seq = sra_its_[pt.tid_]->getFragmentBases();
-			const ngs::StringRef rb_qual = sra_its_[pt.tid_]->getFragmentQualities();
+		if(read_iter_->nextFragment()) {
+			const ngs::StringRef rb_seq = read_iter_->getFragmentBases();
+			const ngs::StringRef rb_qual = read_iter_->getFragmentQualities();
 			readbuf[readi].readOrigBuf.append('\t');
 			readbuf[readi].readOrigBuf.append(rb_seq.data(), rb_seq.size());
 			readbuf[readi].readOrigBuf.append('\t');
 			readbuf[readi].readOrigBuf.append(rb_qual.data(), rb_qual.size());
 		}
 		readbuf[readi].readOrigBuf.append('\n');
+		readi++;
 	}
 
 	pt.setReadId(readCnt_);
@@ -1877,26 +1889,14 @@ void SRAPatternSource::open() {
 			return;
 		}
 
-		size_t window_size = MAX_ROW / sra_its_.size();
-		size_t remainder = MAX_ROW % sra_its_.size();
-		size_t i = 0, start = 1;
+		size_t start = 1;
 
 		if (pp_.skip > 0) {
 			start = pp_.skip + 1;
 			readCnt_ = pp_.skip;
 		}
 
-		while (i < sra_its_.size()) {
-			sra_its_[i] = new ngs::ReadIterator(sra_run.getReadRange(start, window_size, ngs::Read::all));
-			assert(sra_its_[i] != NULL);
-
-			i++;
-			start += window_size;
-			if (i == sra_its_.size() - 1) {
-				window_size += remainder;
-			}
-		}
-
+		read_iter_ = new ngs::ReadIterator(sra_run.getReadRange(start, MAX_ROW, ngs::Read::all));
 	} catch(...) {
 		cerr << "Warning: Could not access \"" << sra_acc << "\" for reading; skipping..." << endl;
 	}
