@@ -31,9 +31,38 @@
 #include "filebuf.h"
 #include "word_io.h"
 #include "ds.h"
+#include "endian_swap.h"
 
 using namespace std;
 
+class RefTooLongException : public exception {
+
+public:
+	RefTooLongException() {
+#ifdef BOWTIE_64BIT_INDEX
+		// This should never happen!
+		msg = "Error: Reference sequence has more than 2^64-1 characters!  "
+		      "Please divide the reference into smaller chunks and index each "
+			  "independently.";
+#else
+		msg = "Error: Reference sequence has more than 2^32-1 characters!  "
+		      "Please build a large index by passing the --large-index option "
+			  "to bowtie2-build";
+#endif
+	}
+	
+	~RefTooLongException() throw() {}
+	
+	const char* what() const throw() {
+		return msg.c_str();
+	}
+
+protected:
+	
+	string msg;
+	
+};
+
 /**
  * Encapsulates a stretch of the reference containing only unambiguous
  * characters.  From an ordered list of RefRecords, one can (almost)
@@ -43,46 +72,33 @@ using namespace std;
  */
 struct RefRecord {
 	RefRecord() : off(), len(), first() { }
-	RefRecord(uint32_t _off, uint32_t _len, bool _first) :
+	RefRecord(TIndexOffU _off, TIndexOffU _len, bool _first) :
 		off(_off), len(_len), first(_first)
 	{ }
 
 	RefRecord(FILE *in, bool swap) {
 		assert(in != NULL);
-		if(!fread(&off, 4, 1, in)) {
+		if(!fread(&off, OFF_SIZE, 1, in)) {
 			cerr << "Error reading RefRecord offset from FILE" << endl;
 			throw 1;
 		}
-		if(swap) off = endianSwapU32(off);
-		if(!fread(&len, 4, 1, in)) {
+		if(swap) off = endianSwapU(off);
+		if(!fread(&len, OFF_SIZE, 1, in)) {
 			cerr << "Error reading RefRecord offset from FILE" << endl;
 			throw 1;
 		}
-		if(swap) len = endianSwapU32(len);
+		if(swap) len = endianSwapU(len);
 		first = fgetc(in) ? true : false;
 	}
 
-#ifdef BOWTIE_MM
-	RefRecord(int in, bool swap) {
-		off = readU32(in, swap);
-		len = readU32(in, swap);
-		char c;
-		if(!read(in, &c, 1)) {
-			cerr << "Error reading RefRecord 'first' flag" << endl;
-			throw 1;
-		}
-		first = (c ? true : false);
-	}
-#endif
-
 	void write(std::ostream& out, bool be) {
-		writeU32(out, off, be);
-		writeU32(out, len, be);
+		writeU<TIndexOffU>(out, off, be);
+		writeU<TIndexOffU>(out, len, be);
 		out.put(first ? 1 : 0);
 	}
 
-	uint32_t off; /// Offset of the first character in the record
-	uint32_t len; /// Length of the record
+	TIndexOffU off; /// Offset of the first character in the record
+	TIndexOffU len; /// Length of the record
 	bool   first; /// Whether this record is the first for a reference sequence
 };
 
@@ -121,7 +137,7 @@ fastaRefReadSizes(
 	EList<RefRecord>& recs,
 	const RefReadInParams& rparms,
 	BitpairOutFileBuf* bpout,
-	int& numSeqs);
+	TIndexOff& numSeqs);
 
 extern void
 reverseRefRecords(
@@ -139,7 +155,7 @@ static RefRecord fastaRefReadAppend(
 	FileBuf& in,             // input file
 	bool first,              // true iff this is the first record in the file
 	TStr& dst,               // destination buf for parsed characters
-	size_t& dstoff,          // index of next character in dst to assign
+	TIndexOffU& dstoff,          // index of next character in dst to assign
 	RefReadInParams& rparms, // 
 	string* name = NULL)     // put parsed FASTA name here
 {
@@ -292,7 +308,7 @@ static RefRecord fastaRefReadAppend(
 		size_t nlen = dstoff;
 		dst.reverseWindow(ilen, nlen);
 	}
-	return RefRecord((uint32_t)off, (uint32_t)len, first);
+	return RefRecord((TIndexOffU)off, (TIndexOffU)len, first);
 }
 
 #endif /*ndef REF_READ_H_*/
