@@ -325,10 +325,51 @@ public:
 
 			for (i = 0; i < name.length() && !isspace(name[i]); i++) ;
 			o.append('\t');
-			o.append(name.toZBuf() + i + 1);
+			if (i < name.length()) {
+				const char *comment = name.toZBuf() + i + 1;
+				if (isIllumina(comment))
+					o.append("BC:Z:");
+				o.append(comment);
+			}
 		}
 	}
 
+	bool isIllumina(const char *str) const {
+		const char *start, *end;
+		int field = 0;
+
+		for (start = str, end = str; *end != '\0' && *end != ' '; end++) {
+			if (*end == ':') {
+				switch (field) {
+                                case 0: {
+					char *endptr;
+					long read = strtol(start, &endptr, 10);
+					if (endptr != end || (read != 1 && read != 2))
+						return false;
+					break;
+                                }
+                                case 1:
+					if (*start != 'N' && *start != 'Y')
+						return false;
+					break;
+				case 2: {
+					char *endptr;
+					long controlNumber = strtol(start, &endptr, 10);
+					if (endptr != end || controlNumber % 2 != 0)
+						return false;
+					break;
+
+				}
+				default:
+					return false;
+				}
+				start = end + 1, field++;
+			}
+
+		}
+		return true;
+	}
+
 	template<typename T>
 	static void readTagVal(BTString& o, const char *data, size_t &offset, size_t count) {
 		std::vector<T> val(count);
