@@ -20,12 +20,8 @@
 #ifndef UTIL_H_
 #define UTIL_H_
 
-#include <algorithm>
-#include <limits>
 #include <stdlib.h>
-#include <vector>
-
-#include "assert_helpers.h"
+#include <limits>
 
 /**
  * C++ version char* style "itoa":
@@ -46,20 +42,11 @@ char* itoa10(const T& value, char* result) {
 	} while (quotient > 0);
 	// Only apply negative sign for base 10
 	if(std::numeric_limits<T>::is_signed) {
-		// Avoid compiler warning in cases where T is unsigned
 		if (value < 0) *out++ = '-';
 	}
-	std::reverse( result, out );
+	reverse( result, out );
 	*out = 0; // terminator
 	return out;
 }
 
-template <typename Container>
-void erase(Container& v, size_t index) {
-	assert_lt(index, v.size());
-	for (size_t i = index; i < v.size()-1; i++)
-		v[i] = v[i+1];
-	v.pop_back();
-}
-
 #endif /*ndef UTIL_H_*/
